/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hongxi.whatsmars.common.logging;

import org.hongxi.whatsmars.common.logging.inner.Level;
import org.hongxi.whatsmars.common.logging.inner.Logger;
import org.hongxi.whatsmars.common.logging.inner.LoggingEvent;
import org.junit.After;
import org.junit.Before;

import java.io.*;

public class BasicLoggerTest {

    protected Logger logger = Logger.getLogger("test");

    protected LoggingEvent loggingEvent;

    protected String loggingDir = System.getProperty("user.home") + "/logs/rocketmq-test";

    @Before
    public void createLoggingEvent() {
        loggingEvent = new LoggingEvent(Logger.class.getName(), logger, Level.INFO,
            "junit test error", new RuntimeException("createLogging error"));
    }

    public String readFile(String file) throws IOException {
        StringBuilder stringBuilder = new StringBuilder();
        FileInputStream fileInputStream = new FileInputStream(file);
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
        String line = bufferedReader.readLine();
        while (line != null) {
            stringBuilder.append(line);
            stringBuilder.append("\r\n");
            line = bufferedReader.readLine();
        }
        bufferedReader.close();
        return stringBuilder.toString();
    }

    @After
    public void clean() {
        File file = new File(loggingDir);
        if (file.exists()) {
            File[] files = file.listFiles();
            for (File file1 : files) {
                file1.delete();
            }
        }
    }
}
